@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018 Free Software Foundation, Inc.                        */
+/*  Copyright (C) 2018-2019 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -16,12 +16,15 @@
  *
  * A sample program to find the most likely
  * fields for all unknown dwg entities and objects.
- * gather all binary raw data from all unknown dwg entities and objects
+ * Gather all binary raw data from all unknown dwg entities and objects
  * into examples/alldwg.inc and examples/alldxf_*.inc
- * with the available likely fields try permutations of most likely types.
+ * With the available likely fields try permutations of most likely types.
  * When no identifiable field value was found, (filled is empty) 0.0% is
  * printed and the entity is printed to stderr for alldwg.skip to be ignored
  * later.
+ * See also HACKING "reverse-engineering with examples/unknown"
+ * or https://savannah.gnu.org/forum/forum.php?forum_id=9197 and
+ * https://savannah.gnu.org/forum/forum.php?forum_id=9203.
  */
 
 #include "../src/config.h"
@@ -37,14 +40,9 @@
 #include "../src/common.h"
 #include "../src/myalloca.h"
 
-#ifndef M_PI_2
-#  define M_PI_2 1.57079632679489661923132169163975144
-#endif
-#define rad2deg(ang) (ang) * 90.0 / M_PI_2
-#define deg2rad(ang) (ang) ? M_PI_2 / ((ang)*90.0) : 0.0
-
-void *memmem (const void *big, size_t big_len, const void *little,
-              size_t little_len);
+// not needed anymore
+//void *memmem (const void *big, size_t big_len, const void *little,
+//              size_t little_len);
 
 int cur_hdl; // to avoid dupl. search
 
@@ -53,11 +51,11 @@ struct _unknown_field
   int code;
   const char *value;
   unsigned char *bytes;
-  int bitsize;
+  int num_bits;
   Dwg_Bits type;
   const char *name; // in dwg.spec
-  unsigned short
-      num;    // number of occurrences of this type:value pair (max 1423)
+  // number of occurrences of this type:value pair (max 1423)
+  unsigned short num;
   int pos[5]; // 5x found bit offset in dxf->bytes or -1 if not found
   // many typical values are 5x found (handle 0, BL 2)
 };
@@ -67,25 +65,28 @@ static struct _unknown_dxf
   const char *dxf;
   const unsigned int handle;
   const char *bytes;
-  const int bitsize;
+  const int is_entity;
+  const int num_bits; // size of dumped unknown_bits TF
   const int commonsize;
   const int hdloff;
   const int strsize;
-  const int objbitsize;
+  const int hdlsize;
+  const int bitsize;
   const struct _unknown_field *fields;
 } unknown_dxf[] = {
 // see log_unknown_dxf.pl
 #include "alldxf_0.inc"
-  { NULL, NULL, 0, "", 0, 0, 0, 0, 0, NULL }
+  { NULL, NULL, 0, "", 0, 0, 0, 0, 0, 0, 0, NULL }
 };
+
 #include "alldxf_1.inc"
 
-// dynamic helper companian
+// dynamic helper companion
 struct _dxf
 {
   unsigned char *found;    // coverage per bit for found 1
   unsigned char *possible; // coverage for mult. finds >1
-  int bitsize;             // copy of unknown_dxf.bitsize
+  int num_bits;            // copy of unknown_dxf.num_bits
   int num_filled;
   int num_empty;
   int num_possible;
@@ -100,14 +101,16 @@ static struct _unknown {
   const char *bits;
   const char *dxf;
   const unsigned int handle;
-  const int bitsize;
+  const int is_entity;
+  const int num_bits;
   const int commonsize;
   const int hdloff;
   const int strsize;
-  const int objbitsize;
+  const int hdlsize;
+  const int bitsize;
 } unknowns[] =
   {
-   { "ACAD_EVALUATION_GRAPH", "example_2000.log", "40501406481013fffffffcffffffff3fffffffcffffffff980c0c80b8bee", "test/test-data/example_2000.dxf", 0x2E3, 234, 60, -60, 0, 268 },
+   { "ACAD_EVALUATION_GRAPH", "example_2000.log", "40501406481013fffffffcffffffff3fffffffcffffffff980c0c80b8bee", "test/test-data/example_2000.dxf", 0x2E3, 0, 234, 60, -60, 0, 268 },
     /* the following types:
       5 ACDBASSOCGEOMDEPENDENCY
       3 ACDBASSOCNETWORK
@@ -123,7 +126,7 @@ static struct _unknown {
     // see log_unknown.pl
 #  include "alldwg.inc"
 
-   { 0, NULL, "", "", NULL, 0L, 0, 0, 0, 0, 0 }
+  { 0, NULL, "", "", NULL, 0L, 0, 0, 0, 0, 0, 0 }
 };
 #endif
 
@@ -137,21 +140,6 @@ static struct _bd
   { NULL, NULL }
 };
 
-static void
-bits_string (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
-{
-  if (dat->version >= R_2007)
-    {
-      bit_write_TU (dat, (BITCODE_TU)g->value);
-      g->type = BITS_TU;
-    }
-  else
-    {
-      bit_write_TV (dat, (char *)g->value);
-      g->type = BITS_TV;
-    }
-}
-
 static void
 bits_TV (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
 {
@@ -159,6 +147,13 @@ bits_TV (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
   g->type = BITS_TV;
 }
 
+static void
+bits_TU (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
+{
+  bit_write_TU (dat, (BITCODE_TU)g->value);
+  g->type = BITS_TU;
+}
+
 static void
 bits_TF (Bit_Chain *restrict dat, struct _unknown_field *restrict g, int len)
 {
@@ -166,6 +161,15 @@ bits_TF (Bit_Chain *restrict dat, struct _unknown_field *restrict g, int len)
   g->type = BITS_TF;
 }
 
+static void
+bits_string (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
+{
+  if (dat->version >= R_2007)
+    bits_TU (dat, g);
+  else
+    bits_TV (dat, g);
+}
+
 static void
 bits_hexstring (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
 {
@@ -301,11 +305,28 @@ bits_RL (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
 static void
 bits_CMC (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
 {
-  // dat should know if >= R_2004, but we just search for the index
-  // we first try EMC, only if it fails CMC
+  // dat should know if >= R_2004, but we just search for the index (62)
+  // we try ENC on entities, CMC on objects
   Dwg_Color color;
   memset (&color, 0, sizeof (color));
   color.index = strtol (g->value, NULL, 10);
+  if (dat->version >= R_2004)
+    {
+      // check next g field
+      struct _unknown_field *ng = g + 1;
+      struct _unknown_field *ng2 = g + 2;
+      // TODO: need to detect book_name (flag 2)
+      if (ng->code >= 420 && ng->code < 430)
+        {
+          color.rgb = strtol (ng->value, NULL, 10);
+          if (ng2->code >= 430 && ng2->code < 440)
+            {
+              color.flag |= 0x1;
+              color.name = (char *)ng2->value;
+              ng2++;
+            }
+        }
+    }
   bit_write_CMC (dat, &color);
   g->type = BITS_CMC;
 }
@@ -351,7 +372,7 @@ bits_ENC (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
           color.alpha = strtol (ng->value, NULL, 10);
         }
     }
-  bit_write_ENC (dat, &color);
+  bit_write_ENC (dat, dat, dat, &color);
   g->type = BITS_ENC;
 }
 
@@ -361,7 +382,7 @@ handle_string (char *restrict dest, const int code, const char *restrict hdl,
 {
   Dwg_Handle handle;
   // parse hex -> owner handle;
-  sscanf (hdl, "%lX", &handle.value);
+  sscanf (hdl, "%X", &handle.value);
   handle.code = code;
   if (code > 5)
     { // relative offset to objhandle
@@ -393,7 +414,7 @@ handle_string (char *restrict dest, const int code, const char *restrict hdl,
     handle.size = 3;
   else
     handle.size = 4;
-  sprintf (dest, "%x.%d.%lX", code, handle.size, handle.value);
+  sprintf (dest, FORMAT_H, ARGS_H (handle));
 }
 
 static void
@@ -402,7 +423,7 @@ bits_handle (Bit_Chain *restrict dat, struct _unknown_field *restrict g,
 {
   Dwg_Handle handle;
   // parse hex -> owner handle;
-  sscanf (g->value, "%lX", &handle.value);
+  sscanf (g->value, "%X", &handle.value);
   handle.code = code;
   cur_hdl = code;
   if (code > 5)
@@ -435,8 +456,7 @@ bits_handle (Bit_Chain *restrict dat, struct _unknown_field *restrict g,
     handle.size = 3;
   else
     handle.size = 4;
-  printf ("  handle %x.%d.%lX (%X)\n", code, handle.size, handle.value,
-          objhandle);
+  printf ("  handle " FORMAT_H " (%X)\n", ARGS_H (handle), objhandle);
   bit_write_H (dat, &handle);
   g->type = BITS_HANDLE;
 }
@@ -457,14 +477,26 @@ bits_try_handle (struct _unknown_field *g, int code, unsigned int objhandle)
   bits_handle (&dat, g, code, objhandle);
 
   g->bytes = dat.chain;
-  g->bitsize = (dat.byte * 8) + dat.bit;
+  g->num_bits = (dat.byte * 8) + dat.bit;
+}
+
+static int
+dxf_is16 (struct _unknown_dxf *dxf)
+{
+  return strstr (dxf->dxf, "/2007/") || strstr (dxf->dxf, "_2007.dxf")
+                 || strstr (dxf->dxf, "/201") || strstr (dxf->dxf, "_201")
+             ? 1
+             : 0;
 }
 
 static void
-bits_format (struct _unknown_field *g, const int version)
+bits_format (struct _unknown_field *g, const int version,
+             struct _unknown_dxf *dxf)
 {
   int code = g->code;
   Bit_Chain dat = { NULL, 16, 0, 0, NULL, 0, 0 };
+  const int is16 = version >= 2007 ? 1 : 0;
+
   dat.chain = calloc (1, 16);
   if (version)
     {
@@ -485,10 +517,12 @@ bits_format (struct _unknown_field *g, const int version)
   else if (code < 60)
     bits_angle_BD (&dat, g); // deg2rad for angles
   else if (code < 70)
-    if (version >= R_2004)
-      bits_ENC (&dat, g);
-    else
-      bits_CMC (&dat, g);
+    {
+      if (dat.version >= R_2004 && dxf->is_entity)
+        bits_ENC (&dat, g);
+      else
+        bits_CMC (&dat, g);
+    }
   else if (code < 80)
     bits_BS (&dat, g);
   else if (80 <= code && code <= 99) // BL int32
@@ -512,7 +546,7 @@ bits_format (struct _unknown_field *g, const int version)
   else if (code <= 299)
     bits_B (&dat, g);
   else if (code <= 309)
-    bits_TV (&dat, g);
+    bits_string (&dat, g);
   else if (code <= 319)
     bits_hexstring (&dat, g);
   else if (code >= 320 && code < 360)
@@ -561,7 +595,7 @@ bits_format (struct _unknown_field *g, const int version)
   if (g->type)
     {
       g->bytes = dat.chain;
-      g->bitsize = (dat.byte * 8) + dat.bit;
+      g->num_bits = (dat.byte * 8) + dat.bit;
     }
   else
     {
@@ -596,15 +630,15 @@ set_found (struct _dxf *dxf, const struct _unknown_field *g)
 {
   // check for overlap, if already found by some other field
   int overlap = 0;
-  if (g->bitsize + g->pos[0] > dxf->bitsize)
+  if (g->num_bits + g->pos[0] > dxf->num_bits)
     {
       fprintf (stderr, "overflow with found group %d %s: %d+%d >= %d\n",
-               g->code, g->name, g->bitsize, g->pos[0], dxf->bitsize);
+               g->code, g->name, g->num_bits, g->pos[0], dxf->num_bits);
       return 1;
     }
-  for (int k = g->pos[0]; k < g->bitsize + g->pos[0]; k++)
+  for (int k = g->pos[0]; k < g->num_bits + g->pos[0]; k++)
     {
-      if (dxf->found[k] && !overlap && k < g->bitsize)
+      if (dxf->found[k] && !overlap && k < g->num_bits)
         {
           overlap = 1;
           printf ("position %d already found\n", k);
@@ -619,13 +653,13 @@ set_found_i (struct _dxf *dxf, const struct _unknown_field *g, int i)
 {
   // check for overlap, if already found by some other field
   int overlap = 0;
-  if (g->bitsize + g->pos[i] > dxf->bitsize)
+  if (g->num_bits + g->pos[i] > dxf->num_bits)
     {
       fprintf (stderr, "overflow with found group %d %s: %d+%d >= %d\n",
-               g->code, g->name, g->bitsize, g->pos[i], dxf->bitsize);
+               g->code, g->name, g->num_bits, g->pos[i], dxf->num_bits);
       return 1;
     }
-  for (int k = g->pos[i]; k < g->bitsize + g->pos[i]; k++)
+  for (int k = g->pos[i]; k < g->num_bits + g->pos[i]; k++)
     {
       if (dxf->found[k] && !overlap)
         {
@@ -642,13 +676,13 @@ set_possible_pos (struct _dxf *dxf, const struct _unknown_field *g,
                   const int pos)
 {
   // add coverage counter for each bit
-  if (g->bitsize + pos > dxf->bitsize)
+  if (g->num_bits + pos > dxf->num_bits)
     {
       fprintf (stderr, "overflow with possible group %d %s: %d+%d >= %d\n",
-               g->code, g->name, g->bitsize, pos, dxf->bitsize);
+               g->code, g->name, g->num_bits, pos, dxf->num_bits);
       return;
     }
-  for (int k = pos; k < g->bitsize + pos; k++)
+  for (int k = pos; k < g->num_bits + pos; k++)
     {
       dxf->possible[k]++;
     }
@@ -660,7 +694,7 @@ set_possible(struct _dxf *dxf, const struct _unknown_field *g, const int i)
 {
   // add coverage counter for each bit
   for (int j=0; j<i; j++) {
-    for (int k=g->pos[j]; k < g->bitsize + g->pos[j]; k++) {
+    for (int k=g->pos[j]; k < g->num_bits + g->pos[j]; k++) {
       dxf->possible[k]++;
     }
   }
@@ -706,21 +740,21 @@ search_bits (int j, struct _unknown_field *g, struct _unknown_dxf *udxf,
   unsigned char *s;
   unsigned char *found;
   int num_found = 0;
-  int dxf_bitsize = udxf->bitsize;
-  int dxf_size = dxf_bitsize / 8;
+  int dxf_num_bits = udxf->num_bits;
+  int dxf_size = dxf_num_bits / 8;
 
-  if (!g->type || !g->bitsize || !dxf_size)
+  if (!g->type || !g->num_bits || !dxf_size)
     return 0;
-  size = (g->bitsize / 8) + (g->bitsize % 8 ? 1 : 0);
+  size = (g->num_bits / 8) + (g->num_bits % 8 ? 1 : 0);
   dxf_size += (dxf_size % 8 ? 1 : 0);
   if (size > dxf_size)
     return 0;
   s = alloca (size);
-  printf ("  search %d bits of type %s [%d] in %d:%d ", g->bitsize,
-          dwg_bits_name[g->type], j, dxf_bitsize - offset, offset);
-  bit_print_bits (g->bytes, g->bitsize);
-  while ((offset = membits ((unsigned char *)udxf->bytes, udxf->bitsize,
-                            g->bytes, g->bitsize, offset))
+  printf ("  search %d bits of type %s [%d] in %d:%d ", g->num_bits,
+          dwg_bits_name[g->type], j, dxf_num_bits - offset, offset);
+  bit_print_bits (g->bytes, g->num_bits);
+  while ((offset = membits ((unsigned char *)udxf->bytes, udxf->num_bits,
+                            g->bytes, g->num_bits, offset))
          != -1)
     {
       if (num_found < 5) // record only the first 5 offsets for the solver
@@ -773,7 +807,9 @@ main (int argc, char *argv[])
   char *classes[MAX_CLASSES]; // create files per classes
   struct _dxf *dxf = calloc (sizeof (unknown_dxf) / sizeof (unknown_dxf[0]),
                              sizeof (struct _dxf));
-#include "alldxf_2.inc"
+// clang-format off
+  #include "alldxf_2.inc"
+  // clang-format on
 
   if (argc > 2 && !strcmp (argv[i], "--class"))
     {
@@ -841,13 +877,10 @@ main (int argc, char *argv[])
         {
           int num_fields;
           int num_found = -1;
-          int size = unknown_dxf[i].bitsize;
+          int size = unknown_dxf[i].num_bits;
           struct _unknown_field *g
               = (struct _unknown_field *)unknown_dxf[i].fields;
-          const int is16 = strstr (unknown_dxf[i].dxf, "/2007/")
-                           || strstr (unknown_dxf[i].dxf, "/201")
-                           || strstr (unknown_dxf[i].dxf, "_2007.dxf")
-                           || strstr (unknown_dxf[i].dxf, "_201");
+          const int is16 = dxf_is16 (&unknown_dxf[i]);
           int have_struct = 0;
           int is_dict = 0;
           int is_react = 0;
@@ -865,9 +898,15 @@ main (int argc, char *argv[])
           s = strstr (unknown_dxf[i].dxf, "20");
           if (s)
             sscanf (s, "%d", &version);
-          dxf[i].found = calloc (1, unknown_dxf[i].bitsize + 1);
-          dxf[i].possible = calloc (1, unknown_dxf[i].bitsize + 1);
-          dxf[i].bitsize = unknown_dxf[i].bitsize + 1;
+          else
+            {
+              s = strstr (unknown_dxf[i].dxf, "_r");
+              if (s)
+                sscanf (s + 2, "%d", &version);
+            }
+          dxf[i].found = calloc (1, unknown_dxf[i].num_bits + 1);
+          dxf[i].possible = calloc (1, unknown_dxf[i].num_bits + 1);
+          dxf[i].num_bits = unknown_dxf[i].num_bits + 1;
           // TODO offline: find the shortest objects.
           printf ("\n%s: 0x%X (%d) %s\n", unknown_dxf[i].name,
                   unknown_dxf[i].handle, size, unknown_dxf[i].dxf);
@@ -879,12 +918,12 @@ main (int argc, char *argv[])
                    "  Class=\"%s\",\n"
                    "  Dxf=\"%s\",\n"
                    "  Version=%d,\n"
-                   "  Offsets=[%d, %d, %d, %d], %% hdloff, strsize, "
-                   "commonsize, bitsize\n",
+                   "  Offsets=[%d, %d, %d, %d, %d], %% hdloff, strsize, "
+                   "commonsize, bitsize, hdlsize\n",
                    k, class, unknown_dxf[i].handle, size, unknown_dxf[i].dxf,
                    class, unknown_dxf[i].dxf, version, unknown_dxf[i].hdloff,
                    unknown_dxf[i].strsize, unknown_dxf[i].commonsize,
-                   unknown_dxf[i].objbitsize);
+                   unknown_dxf[i].bitsize, unknown_dxf[i].hdlsize);
           fprintf (pi, "  S=\"");
           bit_fprint_bits (pi, (unsigned char *)unknown_dxf[i].bytes, size);
           fprintf (pi, "\",\n"
@@ -915,7 +954,7 @@ main (int argc, char *argv[])
                 }
               // if we came here from continue, i.e. not_found
               // store the binary repr
-              bits_format (&g[j], version);
+              bits_format (&g[j], version, &unknown_dxf[i]);
             SEARCH:
               // searching for it in the stream and store found position if
               // found only once
@@ -969,7 +1008,7 @@ main (int argc, char *argv[])
 
                       bits_CMC (&dat, &g[j]);
                       g[j].bytes = dat.chain;
-                      g[j].bitsize = (dat.byte * 8) + dat.bit;
+                      g[j].num_bits = (dat.byte * 8) + dat.bit;
                       num_found = search_bits (j, &g[j], &unknown_dxf[i],
                                                &dxf[i], offset);
                       free (dat.chain);
@@ -985,7 +1024,7 @@ main (int argc, char *argv[])
 
                       bits_RC (&dat, &g[j]);
                       g[j].bytes = dat.chain;
-                      g[j].bitsize = (dat.byte * 8) + dat.bit;
+                      g[j].num_bits = (dat.byte * 8) + dat.bit;
                       num_found = search_bits (j, &g[j], &unknown_dxf[i],
                                                &dxf[i], offset);
                       free (dat.chain);
@@ -1001,7 +1040,7 @@ main (int argc, char *argv[])
 
                       bits_BL (&dat, &g[j]);
                       g[j].bytes = dat.chain;
-                      g[j].bitsize = (dat.byte * 8) + dat.bit;
+                      g[j].num_bits = (dat.byte * 8) + dat.bit;
                       num_found = search_bits (j, &g[j], &unknown_dxf[i],
                                                &dxf[i], offset);
                       if (num_found)
@@ -1013,7 +1052,7 @@ main (int argc, char *argv[])
                       bit_set_position (&dat, 0);
                       bits_RS (&dat, &g[j]);
                       g[j].bytes = dat.chain;
-                      g[j].bitsize = (dat.byte * 8) + dat.bit;
+                      g[j].num_bits = (dat.byte * 8) + dat.bit;
                       num_found = search_bits (j, &g[j], &unknown_dxf[i],
                                                &dxf[i], offset);
                       free (dat.chain);
@@ -1030,7 +1069,7 @@ main (int argc, char *argv[])
 
                       bits_BS (&dat, &g[j]);
                       g[j].bytes = dat.chain;
-                      g[j].bitsize = (dat.byte * 8) + dat.bit;
+                      g[j].num_bits = (dat.byte * 8) + dat.bit;
                       num_found = search_bits (j, &g[j], &unknown_dxf[i],
                                                &dxf[i], offset);
                       if (num_found)
@@ -1042,7 +1081,7 @@ main (int argc, char *argv[])
                       bit_set_position (&dat, 0);
                       bits_RL (&dat, &g[j]);
                       g[j].bytes = dat.chain;
-                      g[j].bitsize = (dat.byte * 8) + dat.bit;
+                      g[j].num_bits = (dat.byte * 8) + dat.bit;
                       num_found = search_bits (j, &g[j], &unknown_dxf[i],
                                                &dxf[i], offset);
                       free (dat.chain);
@@ -1058,7 +1097,7 @@ main (int argc, char *argv[])
 
                       bits_BS (&dat, &g[j]);
                       g[j].bytes = dat.chain;
-                      g[j].bitsize = (dat.byte * 8) + dat.bit;
+                      g[j].num_bits = (dat.byte * 8) + dat.bit;
                       num_found = search_bits (j, &g[j], &unknown_dxf[i],
                                                &dxf[i], offset);
                       free (dat.chain);
@@ -1074,7 +1113,7 @@ main (int argc, char *argv[])
 
                       bits_RD (&dat, &g[j]);
                       g[j].bytes = dat.chain;
-                      g[j].bitsize = (dat.byte * 8) + dat.bit;
+                      g[j].num_bits = (dat.byte * 8) + dat.bit;
                       num_found = search_bits (j, &g[j], &unknown_dxf[i],
                                                &dxf[i], offset);
                       free (dat.chain);
@@ -1096,7 +1135,7 @@ main (int argc, char *argv[])
                       g[j].bytes = dat.chain;
 
                       if (dat.byte == 8)
-                        g[j].bitsize = 58; // from 66
+                        g[j].num_bits = 58; // from 66
                       else
                         {
                           free (dat.chain);
@@ -1111,7 +1150,7 @@ main (int argc, char *argv[])
                         {
                           free (dat.chain);
                           dat.chain = (unsigned char *)unknown_dxf[i].bytes;
-                          dat.size = unknown_dxf[i].bitsize / 8;
+                          dat.size = unknown_dxf[i].num_bits / 8;
                           bit_set_position (&dat, g[j].pos[0]);
                           d = bit_read_BD (&dat);
                           if (fabs (d - strtod (g[j].value, NULL)) < 0.001)
@@ -1128,7 +1167,7 @@ main (int argc, char *argv[])
                         }
                       else
                         {
-                          g[j].bitsize = 54; // from 66
+                          g[j].num_bits = 54; // from 66
                           // printf("  more imprecise BD search, 38bit mantissa
                           // precision\n");
                           num_found = search_bits (j, &g[j], &unknown_dxf[i],
@@ -1138,7 +1177,7 @@ main (int argc, char *argv[])
                             {
                               dat.chain
                                   = (unsigned char *)unknown_dxf[i].bytes;
-                              dat.size = unknown_dxf[i].bitsize / 8;
+                              dat.size = unknown_dxf[i].num_bits / 8;
                               bit_set_position (&dat, g[j].pos[0]);
                               d = bit_read_BD (&dat);
                               if (fabs (d - strtod (g[j].value, NULL)) < 0.001)
@@ -1167,7 +1206,7 @@ main (int argc, char *argv[])
                       g[j].bytes = dat.chain;
 
                       if (dat.byte == 8)
-                        g[j].bitsize = 56; // from 64
+                        g[j].num_bits = 56; // from 64
                       else
                         {
                           free (dat.chain);
@@ -1182,7 +1221,7 @@ main (int argc, char *argv[])
                         {
                           free (dat.chain);
                           dat.chain = (unsigned char *)unknown_dxf[i].bytes;
-                          dat.size = unknown_dxf[i].bitsize / 8;
+                          dat.size = unknown_dxf[i].num_bits / 8;
                           bit_set_position (&dat, g[j].pos[0]);
                           d = bit_read_BD (&dat);
                           if (fabs (d - strtod (g[j].value, NULL)) < 0.001)
@@ -1200,7 +1239,7 @@ main (int argc, char *argv[])
                         }
                       else
                         {
-                          g[j].bitsize = 52; // from 64
+                          g[j].num_bits = 52; // from 64
                           // printf("  more imprecise RD search, 38bit mantissa
                           // precision\n");
                           num_found = search_bits (j, &g[j], &unknown_dxf[i],
@@ -1210,7 +1249,7 @@ main (int argc, char *argv[])
                             {
                               dat.chain
                                   = (unsigned char *)unknown_dxf[i].bytes;
-                              dat.size = unknown_dxf[i].bitsize / 8;
+                              dat.size = unknown_dxf[i].num_bits / 8;
                               bit_set_position (&dat, g[j].pos[0]);
                               d = bit_read_BD (&dat);
                               if (fabs (d - strtod (g[j].value, NULL)) < 0.001)
@@ -1238,12 +1277,14 @@ main (int argc, char *argv[])
                       int len = strlen (g[j].value);
                       dat.chain = calloc (1, 16);
 
+#if 0
+                      /* TU/TF cannot be mixed */
                       if (is16)
                         {
                           bits_TV (&dat, &g[j]);
 
                           g[j].bytes = dat.chain;
-                          g[j].bitsize = (dat.byte * 8) + dat.bit;
+                          g[j].num_bits = (dat.byte * 8) + dat.bit;
                           num_found = search_bits (j, &g[j], &unknown_dxf[i],
                                                    &dxf[i], offset);
                           if (num_found)
@@ -1252,7 +1293,7 @@ main (int argc, char *argv[])
                               goto FOUND;
                             }
                         }
-
+#endif
                       if (len)
                         {
                           bit_set_position (&dat, 0);
@@ -1260,7 +1301,7 @@ main (int argc, char *argv[])
                                    len); // search without the final \0
 
                           g[j].bytes = dat.chain;
-                          g[j].bitsize = (dat.byte * 8) + dat.bit;
+                          g[j].num_bits = (dat.byte * 8) + dat.bit;
                           num_found = search_bits (j, &g[j], &unknown_dxf[i],
                                                    &dxf[i], offset);
                           free (dat.chain);
@@ -1280,7 +1321,7 @@ main (int argc, char *argv[])
                       // later could be used as hints for the picat solver
                       fprintf (pi, "    %%new_struct('_%s%d', [\"", piname,
                                g[j].code);
-                      bit_fprint_bits (pi, g[j].bytes, g[j].bitsize);
+                      bit_fprint_bits (pi, g[j].bytes, g[j].num_bits);
                       if (g[j].type == BITS_HANDLE)
                         {
                           char buf[32];
@@ -1315,7 +1356,7 @@ main (int argc, char *argv[])
               fprintf (pi, "    %snew_struct('%s%d', [\"",
                        have_struct ? "," : " ", piname, g[j].code);
               have_struct = 1;
-              bit_fprint_bits (pi, g[j].bytes, g[j].bitsize);
+              bit_fprint_bits (pi, g[j].bytes, g[j].num_bits);
               if (g[j].type == BITS_HANDLE)
                 {
                   char buf[32];
@@ -1350,12 +1391,12 @@ main (int argc, char *argv[])
                     }
                   printf ("+ %d: %s [%s] found 1 at offset %d-%d /%d\n",
                           g[j].code, g[j].value, dwg_bits_name[g[j].type],
-                          g[j].pos[0], g[j].pos[0] + g[j].bitsize - 1, size);
+                          g[j].pos[0], g[j].pos[0] + g[j].num_bits - 1, size);
                   if (g[j].num > 1)
                     printf ("        but we have %d same DXF fields\n",
                             g[j].num);
                   else
-                    dxf[i].num_filled += g[j].bitsize;
+                    dxf[i].num_filled += g[j].num_bits;
                 }
               else if (num_found == 2)
                 {
@@ -1363,8 +1404,8 @@ main (int argc, char *argv[])
                       "%s %d: %s [%s] found 2 at offsets %d-%d, %d-%d /%d\n",
                       2 == g[j].num ? "+" : "?", g[j].code, g[j].value,
                       dwg_bits_name[g[j].type], g[j].pos[0],
-                      g[j].pos[0] + g[j].bitsize - 1, g[j].pos[1],
-                      g[j].pos[1] + g[j].bitsize - 1, size);
+                      g[j].pos[0] + g[j].num_bits - 1, g[j].pos[1],
+                      g[j].pos[1] + g[j].num_bits - 1, size);
                   // check if we have two of those fields, then it's unique
                   // also
                   if (2 == g[j].num)
@@ -1372,7 +1413,7 @@ main (int argc, char *argv[])
                       printf ("        and we have %d same DXF fields\n", 2);
                       set_found (&dxf[i], &g[j]);
                       set_found_i (&dxf[i], &g[j], 1);
-                      dxf[i].num_filled += g[j].bitsize;
+                      dxf[i].num_filled += g[j].num_bits;
                     }
                   else if (g[j].num > 1)
                     {
@@ -1386,15 +1427,15 @@ main (int argc, char *argv[])
                       "%s %d: %s [%s] found 3 at offsets %d-%d, %d, %d /%d\n",
                       3 == g[j].num ? "+" : "?", g[j].code, g[j].value,
                       dwg_bits_name[g[j].type], g[j].pos[0],
-                      g[j].pos[0] + g[j].bitsize - 1, g[j].pos[1], g[j].pos[2],
-                      size);
+                      g[j].pos[0] + g[j].num_bits - 1, g[j].pos[1],
+                      g[j].pos[2], size);
                   if (3 == g[j].num)
                     {
                       printf ("        and we have %d same DXF fields\n", 3);
                       set_found (&dxf[i], &g[j]);
                       set_found_i (&dxf[i], &g[j], 1);
                       set_found_i (&dxf[i], &g[j], 2);
-                      dxf[i].num_filled += g[j].bitsize;
+                      dxf[i].num_filled += g[j].num_bits;
                     }
                   else if (g[j].num > 1)
                     {
@@ -1408,7 +1449,7 @@ main (int argc, char *argv[])
                           "%d /%d\n",
                           4 == g[j].num ? "+" : "?", g[j].code, g[j].value,
                           dwg_bits_name[g[j].type], g[j].pos[0],
-                          g[j].pos[0] + g[j].bitsize - 1, g[j].pos[1],
+                          g[j].pos[0] + g[j].num_bits - 1, g[j].pos[1],
                           g[j].pos[2], g[j].pos[3], size);
                   if (4 == g[j].num)
                     {
@@ -1417,7 +1458,7 @@ main (int argc, char *argv[])
                       set_found_i (&dxf[i], &g[j], 1);
                       set_found_i (&dxf[i], &g[j], 2);
                       set_found_i (&dxf[i], &g[j], 3);
-                      dxf[i].num_filled += g[j].bitsize;
+                      dxf[i].num_filled += g[j].num_bits;
                     }
                   else if (g[j].num > 1)
                     {
@@ -1431,7 +1472,7 @@ main (int argc, char *argv[])
                           "%d, %d /%d\n",
                           5 == g[j].num ? "+" : "?", g[j].code, g[j].value,
                           dwg_bits_name[g[j].type], g[j].pos[0],
-                          g[j].pos[0] + g[j].bitsize - 1, g[j].pos[1],
+                          g[j].pos[0] + g[j].num_bits - 1, g[j].pos[1],
                           g[j].pos[2], g[j].pos[3], g[j].pos[4], size);
                   if (5 == g[j].num)
                     {
@@ -1441,7 +1482,7 @@ main (int argc, char *argv[])
                       set_found_i (&dxf[i], &g[j], 2);
                       set_found_i (&dxf[i], &g[j], 3);
                       set_found_i (&dxf[i], &g[j], 4);
-                      dxf[i].num_filled += g[j].bitsize;
+                      dxf[i].num_filled += g[j].num_bits;
                     }
                   else if (g[j].num > 1)
                     {
@@ -1455,7 +1496,7 @@ main (int argc, char *argv[])
                           "%d, %d, ... /%d\n",
                           g[j].code, g[j].value, dwg_bits_name[g[j].type],
                           num_found, g[j].pos[0],
-                          g[j].pos[0] + g[j].bitsize - 1, g[j].pos[1],
+                          g[j].pos[0] + g[j].num_bits - 1, g[j].pos[1],
                           g[j].pos[2], g[j].pos[3], g[j].pos[4], size);
                   printf ("        and we have %d same DXF fields\n",
                           num_found);
@@ -1464,7 +1505,7 @@ main (int argc, char *argv[])
                   set_found_i (&dxf[i], &g[j], 2);
                   set_found_i (&dxf[i], &g[j], 3);
                   set_found_i (&dxf[i], &g[j], 4);
-                  dxf[i].num_filled += g[j].bitsize;
+                  dxf[i].num_filled += g[j].num_bits;
                 }
               else if (num_found > 5)
                 {
@@ -1472,7 +1513,7 @@ main (int argc, char *argv[])
                           "%d, %d, %d, ... /%d\n",
                           g[j].code, g[j].value, dwg_bits_name[g[j].type],
                           num_found, g[j].pos[0],
-                          g[j].pos[0] + g[j].bitsize - 1, g[j].pos[1],
+                          g[j].pos[0] + g[j].num_bits - 1, g[j].pos[1],
                           g[j].pos[2], g[j].pos[3], g[j].pos[4], size);
                   if (g[j].num > 1)
                     printf ("        but we have %d same DXF fields\n",
@@ -1480,7 +1521,7 @@ main (int argc, char *argv[])
                 }
               // same pos, search for next. but only if it's a bigger hit, not
               // just 1-2 bits
-              /*if (num_found >= 1 && g->bitsize > 2 && set_found(&dxf[i],
+              /*if (num_found >= 1 && g->num_bits > 2 && set_found(&dxf[i],
                 &g[j])) { offset = g[j].pos[0]+1; goto SEARCH;
                 }
                 */
@@ -1505,7 +1546,7 @@ main (int argc, char *argv[])
             {
               fprintf (stderr, "empty %s \"%s\" 0x%X %d\n",
                        unknown_dxf[i].name, unknown_dxf[i].dxf,
-                       unknown_dxf[i].handle, unknown_dxf[i].bitsize);
+                       unknown_dxf[i].handle, unknown_dxf[i].num_bits);
             }
           class_filled += dxf[i].num_filled;
           class_size += size;
