@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2009, 2018 Free Software Foundation, Inc.                  */
+/*  Copyright (C) 2009-2019 Free Software Foundation, Inc.                   */
 /*  Copyright (C) 2010 Thien-Thi Nguyen                                      */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
@@ -29,6 +29,7 @@
 
 #include <dwg.h>
 #include "bits.h"
+#include "escape.h"
 #include "suffix.inc"
 
 #ifndef M_PI
@@ -91,17 +92,21 @@ transform_Y (double y)
 static void
 output_TEXT (Dwg_Object *obj)
 {
+  Dwg_Data *dwg = obj->parent;
   Dwg_Entity_TEXT *text = obj->tio.entity->tio.TEXT;
+  char *escaped;
 
-  /*TODO: Juca, fix it properly: */
-  if (text->text_value[0] == '&')
-    return;
+  if (dwg->header.version >= R_2007)
+    escaped = htmlwescape ((BITCODE_TU)text->text_value);
+  else
+    escaped = htmlescape (text->text_value, (int)dwg->header.codepage);
 
   printf ("\t<text id=\"dwg-object-%d\" x=\"%f\" y=\"%f\" "
           "font-family=\"Verdana\" font-size=\"%f\" fill=\"blue\">%s</text>\n",
           obj->index, transform_X (text->insertion_pt.x),
           transform_Y (text->insertion_pt.y), text->height /* fontsize */,
-          text->text_value);
+          escaped ? escaped : "");
+  free (escaped);
 }
 
 static void
@@ -149,20 +154,18 @@ output_INSERT (Dwg_Object *obj)
   if (insert->block_header && insert->block_header->handleref.value)
     {
       printf ("\t<use id=\"dwg-object-%d\" transform=\"translate(%f %f) "
-              "rotate(%f) scale(%f %f)\" xlink:href=\"#symbol-%lu\" />"
-              "<!-- block_header->handleref: %d.%d.%lu -->\n",
+              "rotate(%f) scale(%f %f)\" xlink:href=\"#symbol-%lX\" />"
+              "<!-- block_header->handleref: " FORMAT_H " -->\n",
               obj->index, transform_X (insert->ins_pt.x),
               transform_Y (insert->ins_pt.y),
               (180.0 / M_PI) * insert->rotation, insert->scale.x,
               insert->scale.y, insert->block_header->absolute_ref,
-              insert->block_header->handleref.code,
-              insert->block_header->handleref.size,
-              insert->block_header->handleref.value);
+              ARGS_H (insert->block_header->handleref));
     }
   else
     {
-      printf ("\n\n<!-- WRONG INSERT(%d.%d.%lu) -->\n", obj->handle.code,
-              obj->handle.size, obj->handle.value);
+      printf ("\n\n<!-- WRONG INSERT(" FORMAT_H ") -->\n",
+              ARGS_H (obj->handle));
     }
 }
 
@@ -193,34 +196,38 @@ output_BLOCK_HEADER (Dwg_Object_Ref *ref)
   Dwg_Object *obj;
   Dwg_Object_BLOCK_HEADER *hdr;
 
-  if (!ref)
-    {
-      fprintf (stderr, "Found null object reference. Could not output an SVG "
-                       "symbol for this BLOCK_HEADER\n");
-      return;
-    }
+  if (!ref) // silently ignore empty pspaces
+    return;
   if (!ref->obj)
-    {
-      fprintf (stderr, "Found null ref->obj\n");
-      return;
-    }
-  if (ref->obj->type != DWG_TYPE_BLOCK_HEADER)
+    return;
+  obj = ref->obj;
+  if (obj->type != DWG_TYPE_BLOCK_HEADER)
     {
       fprintf (stderr, "Argument not a BLOCK_HEADER reference\n");
       return;
     }
-
   /* TODO: Review.  (This check avoids a segfault, but it is
      still unclear whether or not the condition is valid.)  */
-  if (!ref->obj->tio.object)
+  if (!obj->tio.object)
     {
-      fprintf (stderr, "Found null ref->obj->tio.object\n");
+      fprintf (stderr, "Found null obj->tio.object\n");
       return;
     }
 
-  hdr = ref->obj->tio.object->tio.BLOCK_HEADER;
-  printf ("\t<g id=\"symbol-%lu\" >\n\t\t<!-- %s -->\n", ref->absolute_ref,
-          hdr->name);
+  hdr = obj->tio.object->tio.BLOCK_HEADER;
+  if (hdr->name)
+    {
+      char *escaped;
+      Dwg_Data *dwg = obj->parent;
+      if (dwg->header.version >= R_2007)
+        escaped = htmlwescape ((BITCODE_TU)hdr->name);
+      else
+        escaped = htmlescape (hdr->name, dwg->header.codepage);
+      printf ("\t<g id=\"symbol-%lX\" >\n\t\t<!-- %s -->\n", ref->absolute_ref,
+              escaped ? escaped : "");
+      if (escaped)
+        free (escaped);
+    }
 
   obj = get_first_owned_entity (ref->obj);
   while (obj)
@@ -237,6 +244,7 @@ output_SVG (Dwg_Data *dwg)
 {
   BITCODE_BS i;
   Dwg_Object *obj;
+  Dwg_Object_Ref *ref;
   Dwg_Object_BLOCK_CONTROL *block_control;
   double dx, dy;
 
@@ -263,12 +271,15 @@ output_SVG (Dwg_Data *dwg)
           ">\n",
           page_width, page_height);
 
-  output_BLOCK_HEADER (dwg_model_space_ref (dwg));
-  output_BLOCK_HEADER (dwg_paper_space_ref (dwg));
+  if ((ref = dwg_model_space_ref (dwg)))
+      output_BLOCK_HEADER (ref);
+  if ((ref = dwg_paper_space_ref (dwg)))
+    output_BLOCK_HEADER (ref);
   printf ("\t<defs>\n");
   for (i = 0; i < dwg->block_control.num_entries; i++)
     {
-      output_BLOCK_HEADER (dwg->block_control.block_headers[i]);
+      if ((ref = dwg->block_control.entries[i]))
+        output_BLOCK_HEADER (ref);
     }
   printf ("\t</defs>\n");
 
