@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018 Free Software Foundation, Inc.                        */
+/*  Copyright (C) 2018-2019 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -25,6 +25,7 @@
 #  define JSON if (0)
 #  define FREE if (0)
 #  define IF_ENCODE_FROM_EARLIER if (0)
+#  define IF_ENCODE_FROM_EARLIER_OR_DXF if (0)
 #  define IF_ENCODE_FROM_PRE_R13 if (0)
 #  define IF_ENCODE_FROM_SINCE_R13 if (0)
 #  define IF_IS_ENCODER 0
@@ -37,16 +38,25 @@
 #  endif
 #  define _DWG_FUNC_N(ACTION, name) dwg_##ACTION##_##name
 #  define DWG_FUNC_N(ACTION, name) _DWG_FUNC_N (ACTION, name)
+#  define _DWG_PRIVATE_N(ACTION, name) dwg_##ACTION##_##name##_private
+#  define DWG_PRIVATE_N(ACTION, name) _DWG_PRIVATE_N (ACTION, name)
 
 #  define SET_PARENT(field, obj)
 #  define SET_PARENT_OBJ(field)
 #  define SET_PARENT_FIELD(field, what_parent, obj)
 
-#  ifndef M_PI_2
-#    define M_PI_2 1.57079632679489661923132169163975144
+#  ifndef IS_FREE
+#    define VALUEOUTOFBOUNDS(field, maxvalue)                                 \
+      if (_obj->field > maxvalue)                                             \
+        {                                                                     \
+          LOG_ERROR ("Invalid %s." #field " %lu", obj->name,                  \
+                     (unsigned long)_obj->field);                             \
+          _obj->field = 0;                                                    \
+          return DWG_ERR_VALUEOUTOFBOUNDS;                                    \
+        }
+#  else
+#    define VALUEOUTOFBOUNDS(field, maxvalue) if (0) { }
 #  endif
-#  define rad2deg(ang) (ang) * 90.0 / M_PI_2
-#  define deg2rad(ang) (ang) ? M_PI_2 / ((ang)*90.0) : 0.0
 
 #endif /* SPEC_H */
 
@@ -81,6 +91,12 @@
 #ifndef FIELDG
 #  define FIELDG(nam, type, dxf) FIELD (nam, type)
 #endif
+#ifndef SUB_FIELD_BSd
+#  define SUB_FIELD_BSd(o, nam, dxf) FIELD_BSd (o.nam, dxf)
+#endif
+#ifndef SUB_FIELD_TU
+#  define SUB_FIELD_TU(o, nam, dxf) FIELD_TU (o.nam, dxf)
+#endif
 #ifndef SUB_FIELD_BL
 #  define SUB_FIELD_T(o, nam, dxf) FIELD_T (o.nam, dxf)
 #  define SUB_FIELD_B(o, nam, dxf) FIELDG (o.nam, B, dxf)
@@ -90,6 +106,7 @@
 #  define SUB_FIELD_BS(o, nam, dxf) FIELDG (o.nam, BS, dxf)
 #  define SUB_FIELD_BL(o, nam, dxf) FIELDG (o.nam, BL, dxf)
 #  define SUB_FIELD_BLx(o, nam, dxf) FIELD_BLx (o.nam, dxf)
+#  define SUB_FIELD_BLd(o, nam, dxf) FIELD_BLd (o.nam, dxf)
 #  define SUB_FIELD_BD(o, nam, dxf) FIELDG (o.nam, BD, dxf)
 #  define SUB_FIELD_RC(o, nam, dxf) FIELDG (o.nam, RC, dxf)
 #  define SUB_FIELD_RS(o, nam, dxf) FIELDG (o.nam, RS, dxf)
@@ -114,8 +131,19 @@
 #endif
 #ifndef FIELD_BLx
 #  define FIELD_BLx(name, dxf) FIELD_BL (name, dxf)
+#endif
+#ifndef FIELD_RLd
+#  define FIELD_RLd(name, dxf) FIELD_RL (name, dxf)
+#endif
+#ifndef FIELD_BLd
+#  define FIELD_BLd(name, dxf) FIELD_BL (name, dxf)
+#endif
+#ifndef FIELD_BSx
 #  define FIELD_BSx(name, dxf) FIELD_BS (name, dxf)
 #endif
+#ifndef FIELD_BSd
+#  define FIELD_BSd(name, dxf) FIELD_BS (name, dxf)
+#endif
 #ifndef FIELD_RCu
 #  define FIELD_RCu(name, dxf) FIELD_RC (name, dxf)
 #endif
@@ -123,10 +151,10 @@
 #  define FIELD_RCd(name, dxf) FIELD_RC (name, dxf)
 #endif
 #ifndef VALUE_BINARY
-#  define VALUE_BINARY(value, dxf, size)
+#  define VALUE_BINARY(value, len, dxf)
 #endif
 #ifndef FIELD_BINARY
-#  define FIELD_BINARY(name, dxf, size) VALUE_BINARY (_obj->name, dxf, size)
+#  define FIELD_BINARY(name, len, dxf) FIELD_TF (name, len, dxf)
 #endif
 #ifndef SUBCLASS
 #  define SUBCLASS(text)
@@ -161,13 +189,16 @@
 #  define ENCODER if (1)
 #  undef DECODER_OR_ENCODER
 #  define DECODER_OR_ENCODER if (1)
-// when writing, check also rewriting from an earlier version and fill in a
-// default then
+// when writing, check also rewriting from an earlier version and fill in
+// defaults then
 #  undef IF_ENCODE_FROM_EARLIER
+#  undef IF_ENCODE_FROM_EARLIER_OR_DXF
 #  undef IF_ENCODE_FROM_PRE_R13
 #  undef IF_ENCODE_FROM_SINCE_R13
 #  define IF_ENCODE_FROM_EARLIER                                              \
     if (dat->from_version && dat->from_version < cur_ver)
+#  define IF_ENCODE_FROM_EARLIER_OR_DXF                                       \
+    if ((dat->from_version && dat->from_version < cur_ver) || dwg->opts & 0x20)
 #  define IF_ENCODE_FROM_PRE_R13                                              \
     if (dat->from_version && dat->from_version < R_13)
 #  define IF_ENCODE_SINCE_R13                                                 \
@@ -225,15 +256,25 @@
 #  endif
 #endif
 
-#define R11OPTS(b) _ent->opts_r11 &b
-#define R11FLAG(b) _ent->flag_r11 &b
+#ifndef R11OPTS
+#  define R11OPTS(b) _ent->opts_r11 &b
+#  define R11FLAG(b) _ent->flag_r11 &b
+#endif
 
 #define DECODE_UNKNOWN_BITS                                                   \
   DECODER { dwg_decode_unknown (dat, (Dwg_Object * restrict) obj); }          \
   FREE { VALUE_TF (obj->unknown_bits, 0); }
 
+#ifndef START_OBJECT_HANDLE_STREAM
+#  define START_OBJECT_HANDLE_STREAM                                          \
+    START_HANDLE_STREAM;                                                      \
+    VALUE_HANDLE (obj->tio.object->ownerhandle, ownerhandle, 4, 0);           \
+    REACTORS (4);                                                             \
+    XDICOBJHANDLE (3)
+#endif
+
 #ifndef COMMON_TABLE_FLAGS
-#  define COMMON_TABLE_FLAGS(owner, acdbname)                                 \
+#  define COMMON_TABLE_FLAGS(acdbname)                                        \
     PRE (R_13)                                                                \
     {                                                                         \
       FIELD_RC (flag, 70);                                                    \
@@ -263,7 +304,7 @@
     RESET_VER
 
 // Same as above. just _dwg_object_LAYER::flags is short, not RC
-#  define LAYER_TABLE_FLAGS(owner, acdbname)                                  \
+#  define LAYER_TABLE_FLAGS(acdbname)                                         \
     PRE (R_13)                                                                \
     {                                                                         \
       FIELD_CAST (flag, RC, RS, 70);                                          \
@@ -303,39 +344,47 @@
 #  define END_REPEAT_BLOCK }
 #endif
 
+/* REPEAT names:
+  _ adds idx
+  C does no checks
+  N does constant times (else _obj->times)
+  F does not calloc/free
+*/
+
+// unchecked with constant times
 #ifndef REPEAT
 #  define REPEAT_CN(times, name, type)                                        \
     if (_obj->name)                                                           \
       for (rcount1 = 0; rcount1 < (BITCODE_BL)times; rcount1++)
+// checked with constant times
 #  define REPEAT_N(times, name, type)                                         \
-    if (dat->version >= R_2000 && times > 0x1000)                             \
+    if (dat->version >= R_2000 && (BITCODE_BL)times > 0x1000)                 \
       {                                                                       \
-        LOG_ERROR ("Invalid %s." #name " rcount1 %ld", obj->dxfname,          \
+        LOG_ERROR ("Invalid %s." #name " rcount1 %ld", SAFEDXFNAME,           \
                    (long)times);                                              \
         return DWG_ERR_VALUEOUTOFBOUNDS;                                      \
       }                                                                       \
     if (_obj->name)                                                           \
       for (rcount1 = 0; rcount1 < (BITCODE_BL)times; rcount1++)
 
+// checked with var. times
 #  define _REPEAT(times, name, type, idx)                                     \
-    if (dat->version >= R_2000 && _obj->times > 0x1000)                       \
+    if (dat->version >= R_2000 && (BITCODE_BL)_obj->times > 0x1000)           \
       {                                                                       \
-        LOG_ERROR ("Invalid %s." #name " rcount" #idx " %ld", obj->dxfname,   \
+        LOG_ERROR ("Invalid %s." #name " rcount" #idx " %ld", SAFEDXFNAME,    \
                    (long)_obj->times);                                        \
         return DWG_ERR_VALUEOUTOFBOUNDS;                                      \
       }                                                                       \
     if (_obj->name)                                                           \
       for (rcount##idx = 0; rcount##idx < (BITCODE_BL)_obj->times;            \
            rcount##idx++)
+// unchecked with var. times
 #  ifndef _REPEAT_C
 #    define _REPEAT_C(times, name, type, idx)                                 \
       if (_obj->name)                                                         \
         for (rcount##idx = 0; rcount##idx < (BITCODE_BL)_obj->times;          \
              rcount##idx++)
 #  endif
-#  define _REPEAT_N(times, name, type, idx)                                   \
-    if (_obj->name)                                                           \
-      for (rcount##idx = 0; rcount##idx < (BITCODE_BL)times; rcount##idx++)
 #  define REPEAT(times, name, type) _REPEAT (times, name, type, 1)
 #  define REPEAT2(times, name, type) _REPEAT (times, name, type, 2)
 #  define REPEAT3(times, name, type) _REPEAT (times, name, type, 3)
@@ -344,5 +393,29 @@
 #  define REPEAT2_C(times, name, type) _REPEAT_C (times, name, type, 2)
 #  define REPEAT3_C(times, name, type) _REPEAT_C (times, name, type, 3)
 #  define REPEAT4_C(times, name, type) _REPEAT_C (times, name, type, 4)
-
+#endif
+// unchecked with constant times
+#ifndef _REPEAT_CN
+#  define _REPEAT_CN(times, name, type, idx)                                  \
+    if (_obj->name)                                                           \
+      for (rcount##idx = 0; rcount##idx < (BITCODE_BL)times; rcount##idx++)
+#endif
+// not allocating versions:
+// unchecked
+#ifndef _REPEAT_CNF
+#  define _REPEAT_CNF(times, name, type, idx)                                 \
+    if (_obj->name)                                                           \
+      for (rcount##idx = 0; rcount##idx < (BITCODE_BL)times; rcount##idx++)
+#endif
+#ifndef _REPEAT_NF
+// checked
+#  define _REPEAT_NF(times, name, type, idx)                                  \
+    if (dat->version >= R_2000 && times > 0x1000)                             \
+      {                                                                       \
+        LOG_ERROR ("Invalid %s." #name " rcount" #idx " %ld", SAFEDXFNAME,    \
+                   (long)times);                                              \
+        return DWG_ERR_VALUEOUTOFBOUNDS;                                      \
+      }                                                                       \
+    if (_obj->name)                                                           \
+      for (rcount##idx = 0; rcount##idx < (BITCODE_BL)times; rcount##idx++)
 #endif
